/*******************************************************************************
 *
 * Copyright (C) 1999-2000 by Y. Padioleau, G. Cottenceau, P. Garcia,
 *   G. LeGuelvouit, Y. Zemali, S. Crunchant, X. Michard.
 *
 * Please check out README.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 ******************************************************************************/

#ifndef _CELLULEGENERATRICE_H_
#define _CELLULEGENERATRICE_H_


#include "CommunMoteur.h"
#include "CelluleAvecParticule.h"

//*******************************************************************************
/** Modlisation d'une cellule gnratrice.
 *  Cette classe hrite de la classe CelluleAvecParticule. Elle permet
 *  de redfinir le comportement de certaines mthodes afin de prendre en
 *  compte la possibilit de gnrer une nouvelle particule  l'intrieur
 *  de la cellule considre, en remplacement d'une particule qu'elle
 *  accueille.
 *******************************************************************************/

class CelluleGeneratrice : public CelluleAvecParticule
{
public:
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Constructeurs/Destructeur
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  /// Constructeur  partir de la particule que contiendra la cellule, et de la phase utilise pour gnrer de nouvelles particules.
  CelluleGeneratrice(Particule* part, Phase* phase);


  virtual CelluleGeneratrice* clone() const;

  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Autres mthodes
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


  /// Activation de l'change de particules.
  void activerEchange();

  /// Echange effectif de particules.
  Particule* echangeParticule(Particule* part1);

  /// Obtention de la phase gnre.
  Phase * getPhaseGeneree() const;

  /// Obtention de la phase gnre.
  void setPhaseGeneree(Phase* phase)
    {
      _phaseGeneree = phase;
    }

  /// Mise  jour du pointeur vers automate.
  void setAutomate(Automate* aut);

  Automate* getAutomate() const ;

  // ---- cut
  // les choses a declarer dans les fils : le getIoManager
  // et le static _io (different pour chaque sous class si
  // representation differente
  // UGLY je sais pas si ca marche ca
  // ca doit pas je pense
  // faut que l'adresse des fonctions virtuelles 
  // soit les memes !!!!!!!!!!!!!
  virtual ioManager<Cellule>* getIoManager() const
    {
      return (ioManager<Cellule>*) _io;
    }

  // ne pas oublier de le mettre dans le cpp aussi
  static ioManager<CelluleGeneratrice> *_io;

  /// Mthode utilise pour pouvoir debugger le code.
  virtual void afficheur() const;


private:
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Mthodes prives
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  /// Cre la nouvelle particule gnre.
  void creeNouvelleParticuleGeneree();

  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Attributs privs
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  /// Lien vers la phase gnre.
  Phase* _phaseGeneree;

  /// Pointeur vers l'automate.
  Automate *_automate;
  
};
//##############################################################################
//##############################################################################
//##############################################################################
// METHODES INLINES
//##############################################################################
//##############################################################################
//##############################################################################

//##############################################################################
// Public members
//##############################################################################

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructeurs/Destructeur
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//******************************************************************************
/** Description.
 *  
 ******************************************************************************/
inline CelluleGeneratrice::CelluleGeneratrice(Particule * part,Phase * phase) :

  CelluleAvecParticule(part)
{
  _phaseGeneree = phase;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Autres mthodes
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//******************************************************************************
/** Description.
 *  
 ******************************************************************************/
inline Phase * CelluleGeneratrice::getPhaseGeneree() const
{
  return _phaseGeneree;
}

//******************************************************************************
/** Description.
 *  
 ******************************************************************************/
inline void CelluleGeneratrice::setAutomate(Automate* aut)
{
  _automate = aut;
}

//******************************************************************************
/** Description.
 *  
 ******************************************************************************/
inline Automate* CelluleGeneratrice::getAutomate() const
{
  return _automate;
}

//******************************************************************************
/** Description.
 *  
 ******************************************************************************/

inline CelluleGeneratrice* CelluleGeneratrice::clone() const
{
  return new CelluleGeneratrice(*this);
}

#endif



