/*******************************************************************************
 *
 * Copyright (C) 1999-2000 by Y. Padioleau, G. Cottenceau, P. Garcia,
 *   G. LeGuelvouit, Y. Zemali, S. Crunchant, X. Michard.
 *
 * Please check out README.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 ******************************************************************************/

#ifndef _CELLULESANSPARTICULE_H_
#define _CELLULESANSPARTICULE_H_



#include "CommunMoteur.h"
#include "Cellule.h"

//*******************************************************************************
/** Modlisation d'une cellule sans particule.
 *  Cette classe hrite de la classe Cellule. Elle permet
 *  de prciser le comportement commun des cellules sans particules,
 *  mais n'est pas instanciable. On fera hriter de CelluleSansParticule, les
 *  classes CelluleParoi et CelluleHorsScene.
 *
 *  Il existe des mthodes dans cette classe qui sont censes porter sur
 *  des cellules avec particules. Leur dfinition a t remonte dans la
 *  classe Cellule, et leur implmentation dans la classe
 *  CelluleSansParticule soit n'effectuera aucune opration, soit lvera
 *  une exception ; ceci afin de pouvoir appeller ces mthodes directement
 *  sur les lments d'une liste de Cellule, sans se soucier de quelle
 *  Cellule il s'agit.
 *******************************************************************************/

class CelluleSansParticule : public Cellule {

public:
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Exceptions
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  class Exception {
  public:
    /** Exception leve lorsque l'on essaie d'excuter une mthode portant exclusivement
     *  sur les cellules avec particules.
     */
    Exception(string message);
  };


  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Autres mthodes
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  /// Calcul de la requte associe.
  void calculerRequete();

  /// Activation de l'change de particules.
  void activerEchange();

  /// Obtention de la requte prcdemment calcule.
  Requete getRequete() const;

  /// Activation de la particule thoriquementcontenue dans la cellule.
  void activer();

  /// Obtention de l'activit de la particule contenue dans la cellule.
  bool getActivite() const;

  /// Thoriquement, correspond  l'change de particules.
  Particule * echangeParticule(Particule * part);

  virtual Particule * getParticule() const;

  /// Obtention du \latexonly $\phi$ \endlatexonly de la particule cense se trouver dans la cellule.
  virtual float getPhiParticule() const = 0;

  /// Affiche les attributs privs de l'objet.
  void afficheur() const;

  /// Affiche un attribut identifiant l'objet.
  void afficheurReduit() const;

};
//##############################################################################
//##############################################################################
//##############################################################################
// METHODES INLINES
//##############################################################################
//##############################################################################
//##############################################################################

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Exceptions
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//******************************************************************************
/** Description.
 *  
 ******************************************************************************/
inline CelluleSansParticule::Exception::Exception(string message)
{
  DEBUG_MOTEUR(cerr << "[Exception] " << message << endl;)
}

inline Particule* CelluleSansParticule::getParticule() const
{
  throw CelluleSansParticule::Exception("cellule sans particule"); 
}

#endif

